@@ -30,8 +30,8 @@
  * Tests the DimensionLabel API
  */
 
-#include "test/src/helpers.h"
-#include "test/src/vfs_helpers.h"
+#include "test/support/src/helpers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/sm/array_schema/dimension_label_reference.h"
 #include "tiledb/sm/c_api/experimental/tiledb_dimension_label.h"
@@ -78,19 +78,14 @@ class DimensionLabelTestFixture : public TemporaryDirectoryFixture {
    *  * Attributes:
    *    - a: (type=FLOAT64)
    *  * Dimesnion labels:
-   *    - x: (dim_idx=0, type=label_type, domain=label_domain, tile=tile_extent)
+   *    - x: (dim_idx=0, type=label_type)
    *
    * @param array_name Array name relative to the fixture temporary directory.
    * @param label_type The data type of the dimension label.
-   * @param label_domain The domain of the dimension label.
-   * @param tile_extent The tile extent of the dimension label.
    * @returns Full URI to the generated array.
    */
   std::string create_single_label_array(
-      std::string&& array_name,
-      tiledb_datatype_t label_type,
-      void* label_domain,
-      void* tile_extents);
+      std::string&& array_name, tiledb_datatype_t label_type);
 
   /**
    * Creates a sample test array with multiple dimension labels.
@@ -112,10 +107,7 @@ class DimensionLabelTestFixture : public TemporaryDirectoryFixture {
 };
 
 std::string DimensionLabelTestFixture::create_single_label_array(
-    std::string&& array_name,
-    tiledb_datatype_t label_type,
-    void* label_domain,
-    void* label_tile_extent) {
+    std::string&& array_name, tiledb_datatype_t label_type) {
   // Create an array schema
   uint64_t x_domain[2]{0, 63};
   uint64_t x_tile_extent{64};
@@ -140,13 +132,11 @@ std::string DimensionLabelTestFixture::create_single_label_array(
   tiledb_dimension_label_schema_t* dim_label_schema;
   REQUIRE_TILEDB_OK(tiledb_dimension_label_schema_alloc(
       ctx,
-      TILEDB_INCREASING_LABELS,
+      TILEDB_UNORDERED_LABELS,
+      label_type,
       TILEDB_UINT64,
       x_domain,
       &x_tile_extent,
-      label_type,
-      label_domain,
-      label_tile_extent,
       &dim_label_schema));
 
   REQUIRE_TILEDB_OK(tiledb_array_schema_add_dimension_label(
@@ -188,17 +178,13 @@ std::string DimensionLabelTestFixture::create_multi_label_array(
   // Create and add dimension labels to schema.
   tiledb_dimension_label_schema_t* dim_label_schema;
   // l0
-  double l0_domain[] = {-10.0, 10.0};
-  double l0_tile_extent = 4.0;
   REQUIRE_TILEDB_OK(tiledb_dimension_label_schema_alloc(
       ctx,
       TILEDB_INCREASING_LABELS,
+      TILEDB_FLOAT64,
       TILEDB_UINT64,
       x_domain,
       &x_tile_extent,
-      TILEDB_FLOAT64,
-      l0_domain,
-      &l0_tile_extent,
       &dim_label_schema));
   REQUIRE_TILEDB_OK(tiledb_array_schema_add_dimension_label(
       ctx, array_schema, 0, "x", dim_label_schema));
@@ -207,12 +193,10 @@ std::string DimensionLabelTestFixture::create_multi_label_array(
   REQUIRE_TILEDB_OK(tiledb_dimension_label_schema_alloc(
       ctx,
       TILEDB_UNORDERED_LABELS,
+      TILEDB_STRING_ASCII,
       TILEDB_UINT64,
       x_domain,
       &x_tile_extent,
-      TILEDB_STRING_ASCII,
-      nullptr,
-      nullptr,
       &dim_label_schema));
   REQUIRE_TILEDB_OK(tiledb_array_schema_add_dimension_label(
       ctx, array_schema, 0, "id", dim_label_schema));
@@ -236,18 +220,14 @@ TEST_CASE(
   tiledb_ctx_alloc(NULL, &ctx);
   int64_t dim_domain[] = {1, 10};
   int64_t tile_extent = 5;
-  double label_domain[] = {-10.0, 10.0};
-  double label_tile_extent = 4.0;
   tiledb_dimension_label_schema_t* dim_label;
   REQUIRE_TILEDB_OK(tiledb_dimension_label_schema_alloc(
       ctx,
       TILEDB_INCREASING_LABELS,
+      TILEDB_FLOAT64,
       TILEDB_INT64,
       dim_domain,
       &tile_extent,
-      TILEDB_FLOAT64,
-      label_domain,
-      &label_tile_extent,
       &dim_label));
   tiledb_dimension_label_schema_free(&dim_label);
 }
@@ -259,13 +239,7 @@ TEST_CASE_METHOD(
   // Create and add dimension label schema (both fixed and variable length
   // examples).
   auto label_type = GENERATE(TILEDB_FLOAT64, TILEDB_STRING_ASCII);
-  double label_domain[] = {-10.0, 10.0};
-  double label_tile_extent = 4.0;
-  auto array_name = create_single_label_array(
-      "array0",
-      label_type,
-      (label_type == TILEDB_STRING_ASCII) ? nullptr : label_domain,
-      (label_type == TILEDB_STRING_ASCII) ? nullptr : &label_tile_extent);
+  auto array_name = create_single_label_array("array0", label_type);
 
   // Load array schema and check number of labels.
   tiledb_array_schema_t* loaded_array_schema{nullptr};
@@ -287,22 +261,14 @@ TEST_CASE_METHOD(
   URI label_uri{array_name + "/" + x_label_uri};
   DimensionLabel dim_label(label_uri, ctx->storage_manager());
   dim_label.open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0);
-  CHECK(dim_label.index_dimension()->type() == Datatype::UINT64);
-  CHECK(dim_label.index_attribute()->type() == Datatype::UINT64);
-  CHECK(dim_label.label_attribute()->type() == label_datatype);
-  CHECK(dim_label.label_attribute()->type() == label_datatype);
+  CHECK(dim_label.schema().index_type() == Datatype::UINT64);
+  CHECK(dim_label.schema().label_type() == label_datatype);
   const auto& indexed_array_schema =
       dim_label.indexed_array()->array_schema_latest();
   CHECK(indexed_array_schema.domain().dim_num() == 1);
   CHECK(indexed_array_schema.attribute_num() == 1);
   CHECK(indexed_array_schema.dimension_ptr(0)->type() == Datatype::UINT64);
   CHECK(indexed_array_schema.attribute(0)->type() == label_datatype);
-  const auto& labelled_array_schema =
-      dim_label.labelled_array()->array_schema_latest();
-  CHECK(labelled_array_schema.domain().dim_num() == 1);
-  CHECK(labelled_array_schema.attribute_num() == 1);
-  CHECK(labelled_array_schema.dimension_ptr(0)->type() == label_datatype);
-  CHECK(labelled_array_schema.attribute(0)->type() == Datatype::UINT64);
   dim_label.close();
 
   // Free remaining resources
