@@ -31,14 +31,14 @@
  * Tests of C API for (dense or sparse) array operations.
  */
 
+#include <test/support/tdb_catch.h>
 #include "tiledb/sm/c_api/tiledb.h"
 
 #include <iostream>
 
-#include <test/support/tdb_catch.h>
-#include "test/src/helpers.h"
-#include "test/src/serialization_wrappers.h"
-#include "test/src/vfs_helpers.h"
+#include "test/support/src/helpers.h"
+#include "test/support/src/serialization_wrappers.h"
+#include "test/support/src/vfs_helpers.h"
 #ifdef _WIN32
 #if !defined(NOMINMAX)
 #define NOMINMAX
@@ -877,11 +877,11 @@ TEST_CASE_METHOD(
     SECTION("no serialization") {
       serialized_writes = false;
     }
-    SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled global order write") {
       serialized_writes = true;
-#endif
     }
+#endif
   }
 
   SECTION("- with encryption") {
@@ -1459,11 +1459,11 @@ TEST_CASE_METHOD(
     SECTION("no serialization") {
       serialized_writes = false;
     }
-    SECTION("serialization enabled global order write") {
 #ifdef TILEDB_SERIALIZATION
+    SECTION("serialization enabled global order write") {
       serialized_writes = true;
-#endif
     }
+#endif
   }
 
   SECTION("- with encryption") {
@@ -2159,7 +2159,7 @@ TEST_CASE_METHOD(
   // this here by forcing metadata loading
   if (!array_v2) {
     Metadata* metadata = nullptr;
-    array->array_->metadata(&metadata);
+    CHECK(array->array_->metadata(&metadata).ok());
     array->array_->non_empty_domain();
   }
 
