@@ -113,6 +113,8 @@ TEST_CASE("C API: Test enum values", "[capi][enums]") {
   REQUIRE(TILEDB_FILTER_DICTIONARY == 14);
   REQUIRE(TILEDB_FILTER_SCALE_FLOAT == 15);
   REQUIRE(TILEDB_FILTER_XOR == 16);
+  REQUIRE(TILEDB_FILTER_BITSORT == 17);
+  REQUIRE(TILEDB_FILTER_WEBP == 18);
 
   /** Filter option */
   REQUIRE(TILEDB_COMPRESSION_LEVEL == 0);
@@ -443,6 +445,9 @@ TEST_CASE("C API: Test enum string conversion", "[capi][enums]") {
   REQUIRE(
       (tiledb_filter_type_from_str("XOR", &filter_type) == TILEDB_OK &&
        filter_type == TILEDB_FILTER_XOR));
+  REQUIRE(
+      (tiledb_filter_type_from_str("WEBP", &filter_type) == TILEDB_OK &&
+       filter_type == TILEDB_FILTER_WEBP));
 
   tiledb_filter_option_t filter_option;
   REQUIRE(
@@ -481,6 +486,18 @@ TEST_CASE("C API: Test enum string conversion", "[capi][enums]") {
       (tiledb_filter_option_from_str("SCALE_FLOAT_OFFSET", &filter_option) ==
            TILEDB_OK &&
        filter_option == TILEDB_SCALE_FLOAT_OFFSET));
+  REQUIRE(
+      (tiledb_filter_option_from_str("WEBP_QUALITY", &filter_option) ==
+           TILEDB_OK &&
+       filter_option == TILEDB_WEBP_QUALITY));
+  REQUIRE(
+      (tiledb_filter_option_from_str("WEBP_INPUT_FORMAT", &filter_option) ==
+           TILEDB_OK &&
+       filter_option == TILEDB_WEBP_INPUT_FORMAT));
+  REQUIRE(
+      (tiledb_filter_option_from_str("WEBP_LOSSLESS", &filter_option) ==
+           TILEDB_OK &&
+       filter_option == TILEDB_WEBP_LOSSLESS));
 
   tiledb_encryption_type_t encryption_type;
   REQUIRE(
