@@ -262,36 +262,31 @@ Status Attribute::get_nullable(bool* const nullable) {
   return Status::Ok();
 }
 
-Status Attribute::set_filter_pipeline(const FilterPipeline* pipeline) {
-  if (pipeline == nullptr)
-    return LOG_STATUS(Status_AttributeError(
-        "Cannot set filter pipeline to attribute; Pipeline cannot be null"));
-
-  for (unsigned i = 0; i < pipeline->size(); ++i) {
+Status Attribute::set_filter_pipeline(const FilterPipeline& pipeline) {
+  for (unsigned i = 0; i < pipeline.size(); ++i) {
     if (datatype_is_real(type_) &&
-        pipeline->get_filter(i)->type() == FilterType::FILTER_DOUBLE_DELTA)
+        pipeline.get_filter(i)->type() == FilterType::FILTER_DOUBLE_DELTA)
       return LOG_STATUS(
           Status_AttributeError("Cannot set DOUBLE DELTA filter to an "
                                 "attribute with a real datatype"));
   }
 
-  if (type_ == Datatype::STRING_ASCII && var_size() && pipeline->size() > 1) {
-    if (pipeline->has_filter(FilterType::FILTER_RLE) &&
-        pipeline->get_filter(0)->type() != FilterType::FILTER_RLE) {
+  if (type_ == Datatype::STRING_ASCII && var_size() && pipeline.size() > 1) {
+    if (pipeline.has_filter(FilterType::FILTER_RLE) &&
+        pipeline.get_filter(0)->type() != FilterType::FILTER_RLE) {
       return LOG_STATUS(Status_ArraySchemaError(
           "RLE filter must be the first filter to apply when used on a "
           "variable length string attribute"));
     }
-    if (pipeline->has_filter(FilterType::FILTER_DICTIONARY) &&
-        pipeline->get_filter(0)->type() != FilterType::FILTER_DICTIONARY) {
+    if (pipeline.has_filter(FilterType::FILTER_DICTIONARY) &&
+        pipeline.get_filter(0)->type() != FilterType::FILTER_DICTIONARY) {
       return LOG_STATUS(Status_ArraySchemaError(
           "Dictionary filter must be the first filter to apply when used on a "
           "variable length string attribute"));
     }
   }
 
-  filters_ = *pipeline;
-
+  filters_ = pipeline;
   return Status::Ok();
 }
 
