@@ -55,7 +55,7 @@ namespace sm {
 /* ****************************** */
 
 FragmentConsolidator::FragmentConsolidator(
-    const Config* config, StorageManager* storage_manager)
+    const Config& config, StorageManager* storage_manager)
     : Consolidator(storage_manager) {
   auto st = set_config(config);
   if (!st.ok()) {
@@ -109,8 +109,8 @@ Status FragmentConsolidator::consolidate(
       encryption_key,
       key_length);
   if (!st.ok()) {
-    array_for_reads->close();
-    array_for_writes->close();
+    throw_if_not_ok(array_for_reads->close());
+    throw_if_not_ok(array_for_writes->close());
     return st;
   }
 
@@ -129,8 +129,8 @@ Status FragmentConsolidator::consolidate(
         &to_consolidate,
         &union_non_empty_domains);
     if (!st.ok()) {
-      array_for_reads->close();
-      array_for_writes->close();
+      throw_if_not_ok(array_for_reads->close());
+      throw_if_not_ok(array_for_writes->close());
       return st;
     }
 
@@ -148,8 +148,8 @@ Status FragmentConsolidator::consolidate(
         union_non_empty_domains,
         &new_fragment_uri);
     if (!st.ok()) {
-      array_for_reads->close();
-      array_for_writes->close();
+      throw_if_not_ok(array_for_reads->close());
+      throw_if_not_ok(array_for_writes->close());
       return st;
     }
 
@@ -158,8 +158,8 @@ Status FragmentConsolidator::consolidate(
     // consolidated.
     st = fragment_info.load_and_replace(new_fragment_uri, to_consolidate);
     if (!st.ok()) {
-      array_for_reads->close();
-      array_for_writes->close();
+      throw_if_not_ok(array_for_reads->close());
+      throw_if_not_ok(array_for_writes->close());
       return st;
     }
 
@@ -168,7 +168,8 @@ Status FragmentConsolidator::consolidate(
 
   } while (step < config_.steps_);
 
-  RETURN_NOT_OK_ELSE(array_for_reads->close(), array_for_writes->close());
+  RETURN_NOT_OK_ELSE(
+      array_for_reads->close(), throw_if_not_ok(array_for_writes->close()));
   RETURN_NOT_OK(array_for_writes->close());
 
   stats_->add_counter("consolidate_step_num", step);
@@ -217,8 +218,8 @@ Status FragmentConsolidator::consolidate_fragments(
       encryption_key,
       key_length);
   if (!st.ok()) {
-    array_for_reads->close();
-    array_for_writes->close();
+    throw_if_not_ok(array_for_reads->close());
+    throw_if_not_ok(array_for_writes->close());
     return st;
   }
 
@@ -260,8 +261,8 @@ Status FragmentConsolidator::consolidate_fragments(
       union_non_empty_domains,
       &new_fragment_uri);
   if (!st.ok()) {
-    array_for_reads->close();
-    array_for_writes->close();
+    throw_if_not_ok(array_for_reads->close());
+    throw_if_not_ok(array_for_writes->close());
     return st;
   }
 
@@ -270,13 +271,14 @@ Status FragmentConsolidator::consolidate_fragments(
   // consolidated.
   st = fragment_info.load_and_replace(new_fragment_uri, to_consolidate);
   if (!st.ok()) {
-    array_for_reads->close();
-    array_for_writes->close();
+    throw_if_not_ok(array_for_reads->close());
+    throw_if_not_ok(array_for_writes->close());
     return st;
   }
 
-  RETURN_NOT_OK_ELSE(array_for_reads->close(), array_for_writes->close());
-  RETURN_NOT_OK(array_for_writes->close());
+  RETURN_NOT_OK_ELSE(
+      array_for_reads->close(), RETURN_NOT_OK(array_for_writes->close()));
+  throw_if_not_ok(array_for_writes->close());
 
   return Status::Ok();
 }
@@ -401,9 +403,10 @@ Status FragmentConsolidator::consolidate_internal(
     RETURN_NOT_OK(
         utils::parse::get_timestamp_range(to_consolidate[0].uri_, &timestamps));
 
-    for (auto& delete_tile_location :
-         array_for_reads->array_directory().delete_tiles_location()) {
-      if (delete_tile_location.timestamp() >= timestamps.first) {
+    for (auto& delete_and_update_tile_location :
+         array_for_reads->array_directory()
+             .delete_and_update_tiles_location()) {
+      if (delete_and_update_tile_location.timestamp() >= timestamps.first) {
         config_.with_delete_meta_ = true;
         break;
       }
@@ -469,7 +472,7 @@ Status FragmentConsolidator::consolidate_internal(
     auto st2 = storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir);
     (void)st2;  // Perhaps report this once we support an error stack
     if (is_dir)
-      storage_manager_->vfs()->remove_dir(*new_fragment_uri);
+      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
     return st;
   }
 
@@ -479,9 +482,10 @@ Status FragmentConsolidator::consolidate_internal(
     tdb_delete(query_r);
     tdb_delete(query_w);
     bool is_dir = false;
-    storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir);
+    throw_if_not_ok(
+        storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir));
     if (is_dir)
-      storage_manager_->vfs()->remove_dir(*new_fragment_uri);
+      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
     return st;
   }
 
@@ -595,7 +599,7 @@ Status FragmentConsolidator::create_queries(
 
   // Enable consolidation with timestamps on the reader, if applicable.
   if (config_.with_timestamps_ && !dense) {
-    (*query_r)->set_consolidation_with_timestamps();
+    throw_if_not_ok((*query_r)->set_consolidation_with_timestamps());
   }
 
   // Get last fragment URI, which will be the URI of the consolidated fragment
@@ -612,16 +616,17 @@ Status FragmentConsolidator::create_queries(
   *query_w = tdb_new(Query, storage_manager_, array_for_writes, fragment_name);
   RETURN_NOT_OK((*query_w)->set_layout(Layout::GLOBAL_ORDER));
   RETURN_NOT_OK((*query_w)->disable_checks_consolidation());
+  (*query_w)->set_fragment_size(config_.max_fragment_size_);
   if (array_for_reads->array_schema_latest().dense()) {
     RETURN_NOT_OK((*query_w)->set_subarray_unsafe(subarray));
   }
 
   // Set the processed conditions on new fragment.
-  const auto& delete_tiles_location =
-      (*query_r)->array()->array_directory().delete_tiles_location();
+  const auto& delete_and_update_tiles_location =
+      (*query_r)->array()->array_directory().delete_and_update_tiles_location();
   std::vector<std::string> processed_conditions;
-  processed_conditions.reserve(delete_tiles_location.size());
-  for (auto& location : delete_tiles_location) {
+  processed_conditions.reserve(delete_and_update_tiles_location.size());
+  for (auto& location : delete_and_update_tiles_location) {
     processed_conditions.emplace_back(location.condition_marker());
   }
   (*query_w)->set_processed_conditions(processed_conditions);
@@ -842,12 +847,10 @@ Status FragmentConsolidator::set_query_buffers(
   return Status::Ok();
 }
 
-Status FragmentConsolidator::set_config(const Config* config) {
+Status FragmentConsolidator::set_config(const Config& config) {
   // Set the consolidation config for ease of use
   Config merged_config = storage_manager_->config();
-  if (config) {
-    merged_config.inherit(*config);
-  }
+  merged_config.inherit(config);
   bool found = false;
   config_.amplification_ = 0.0f;
   RETURN_NOT_OK(merged_config.get<float>(
@@ -861,6 +864,12 @@ Status FragmentConsolidator::set_config(const Config* config) {
   RETURN_NOT_OK(merged_config.get<uint64_t>(
       "sm.consolidation.buffer_size", &config_.buffer_size_, &found));
   assert(found);
+  config_.max_fragment_size_ = 0;
+  RETURN_NOT_OK(merged_config.get<uint64_t>(
+      "sm.consolidation.max_fragment_size",
+      &config_.max_fragment_size_,
+      &found));
+  assert(found);
   config_.size_ratio_ = 0.0f;
   RETURN_NOT_OK(merged_config.get<float>(
       "sm.consolidation.step_size_ratio", &config_.size_ratio_, &found));
