@@ -85,6 +85,13 @@ inline Status query_type_enum(
   return Status::Ok();
 }
 
+/* Throws error if cell order's enumeration is greater than 4. */
+inline void ensure_query_type_is_valid(QueryType type) {
+  if (type > QueryType::MODIFY_EXCLUSIVE)
+    throw std::runtime_error(
+        "Invalid query type " + std::to_string(static_cast<uint8_t>(type)));
+}
+
 }  // namespace sm
 }  // namespace tiledb
 
