@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -75,14 +75,48 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class Tile;
 
 enum class Filesystem : uint8_t;
 enum class VFSMode : uint8_t;
 
+/** The VFS configuration parameters. */
+struct VFSParameters {
+  VFSParameters() = delete;
+
+  VFSParameters(const Config& config)
+      : max_batch_size_(config.get<uint64_t>("vfs.max_batch_size").value())
+      , min_batch_gap_(config.get<uint64_t>("vfs.min_batch_gap").value())
+      , min_batch_size_(config.get<uint64_t>("vfs.min_batch_size").value())
+      , min_parallel_size_(
+            config.get<uint64_t>("vfs.min_parallel_size").value())
+      , read_ahead_cache_size_(
+            config.get<uint64_t>("vfs.read_ahead_cache_size").value())
+      , read_ahead_size_(config.get<uint64_t>("vfs.read_ahead_size").value()){};
+
+  ~VFSParameters() = default;
+
+  /** The maximum number of bytes in a batched read operation. */
+  uint64_t max_batch_size_;
+
+  /** The minimum number of bytes between two read batches. */
+  uint64_t min_batch_gap_;
+
+  /** The minimum number of bytes in a batched read operation. */
+  uint64_t min_batch_size_;
+
+  /** The minimum number of bytes in a parallel operation. */
+  uint64_t min_parallel_size_;
+
+  /** The byte size of the read-ahead cache. */
+  uint64_t read_ahead_cache_size_;
+
+  /** The byte size to read-ahead for each read. */
+  uint64_t read_ahead_size_;
+};
+
 /**
  * This class implements a virtual filesystem that directs filesystem-related
  * function execution to the appropriate backend based on the input URI.
@@ -114,8 +148,16 @@ class VFS {
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
 
-  /** Constructor. */
-  VFS();
+  /** Constructor.
+   * @param parent_stats The parent stats to inherit from.
+   * @param compute_tp Thread pool for compute-bound tasks.
+   * @param io_tp Thread pool for io-bound tasks.
+   * @param config Configuration parameters.
+   **/
+  VFS(stats::Stats* parent_stats,
+      ThreadPool* compute_tp,
+      ThreadPool* io_tp,
+      const Config& config);
 
   /** Destructor. */
   ~VFS() = default;
@@ -268,29 +310,6 @@ class VFS {
    */
   Status is_empty_bucket(const URI& uri, bool* is_empty) const;
 
-  /**
-   * Initializes the virtual filesystem with the given configuration.
-   *
-   * @param parent_stats The parent stats to inherit from.
-   * @param config Configuration parameters
-   * @return Status
-   */
-  Status init(
-      stats::Stats* parent_stats,
-      ThreadPool* compute_tp,
-      ThreadPool* io_tp,
-      const Config* ctx_config,
-      const Config* vfs_config);
-
-  /**
-   * Terminates the virtual system. Must only be called if init() returned
-   * successfully. The behavior is undefined if not successfully invoked prior
-   * to destructing this object.
-   *
-   * @return Status
-   */
-  Status terminate();
-
   /**
    * Retrieves all the URIs that have the first input as parent.
    *
@@ -696,15 +715,15 @@ class VFS {
   /** The in-memory filesystem which is always supported */
   MemFilesystem memfs_;
 
-  /** Config. */
+  /**
+   * Config.
+   *
+   * Note: This object is stored on the VFS for:
+   * use of API 'tiledb_vfs_get_config'.
+   * pass-by-reference initialization of filesystems' config_ member variables.
+   **/
   Config config_;
 
-  /** `true` if the VFS object has been initialized. */
-  bool init_;
-
-  /** The byte size to read-ahead for each read. */
-  uint64_t read_ahead_size_;
-
   /** The set with the supported filesystems. */
   std::set<Filesystem> supported_fs_;
 
@@ -720,6 +739,9 @@ class VFS {
   /** The read-ahead cache. */
   tdb_unique_ptr<ReadAheadCache> read_ahead_cache_;
 
+  /* The VFS configuration parameters. */
+  VFSParameters vfs_params_;
+
   /* ********************************* */
   /*          PRIVATE METHODS          */
   /* ********************************* */
@@ -781,7 +803,6 @@ class VFS {
   Status max_parallel_ops(const URI& uri, uint64_t* ops) const;
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_VFS_H
