@@ -45,6 +45,7 @@
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/rtree/rtree.h"
+#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 using namespace tiledb::common;
 using namespace tiledb::type;
@@ -60,7 +61,6 @@ class ArraySchema;
 class Buffer;
 class EncryptionKey;
 class MemoryTracker;
-class StorageManager;
 
 /** Stores the metadata structures of a fragment. */
 class FragmentMetadata {
@@ -231,7 +231,7 @@ class FragmentMetadata {
   }
 
   /** Returns the format version of this fragment. */
-  inline uint32_t format_version() const {
+  inline format_version_t format_version() const {
     return version_;
   }
 
@@ -619,10 +619,8 @@ class FragmentMetadata {
 
   /**
    * Compute fragment min, max, sum, null count for all dimensions/attributes.
-   *
-   * @return Status.
    */
-  Status compute_fragment_min_max_sum_null_count();
+  void compute_fragment_min_max_sum_null_count();
 
   /**
    * Sets array schema pointer.
@@ -712,10 +710,9 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, size
+   * @return Size.
    */
-  tuple<Status, optional<uint64_t>> persisted_tile_size(
-      const std::string& name, uint64_t tile_idx);
+  uint64_t persisted_tile_size(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the size of the tile when it is persisted (e.g. the size of the
@@ -724,10 +721,9 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, size
+   * @return Size.
    */
-  tuple<Status, optional<uint64_t>> persisted_tile_var_size(
-      const std::string& name, uint64_t tile_idx);
+  uint64_t persisted_tile_var_size(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the size of the validity tile when it is persisted (e.g. the size
@@ -735,9 +731,9 @@ class FragmentMetadata {
    *
    * @param name The input attribute.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, size
+   * @return Size.
    */
-  tuple<Status, optional<uint64_t>> persisted_tile_validity_size(
+  uint64_t persisted_tile_validity_size(
       const std::string& name, uint64_t tile_idx);
 
   /**
@@ -757,10 +753,9 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, size.
+   * @return Size.
    */
-  tuple<Status, optional<uint64_t>> tile_var_size(
-      const std::string& name, uint64_t tile_idx);
+  uint64_t tile_var_size(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the tile min value for a given attribute or dimension and tile
@@ -768,21 +763,22 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, value, size.
+   * @return Value.
    */
-  tuple<Status, optional<void*>, optional<uint64_t>> get_tile_min(
-      const std::string& name, uint64_t tile_idx);
+  template <typename T>
+  T get_tile_min_as(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the tile max value for a given attribute or dimension and tile
    * index.
    *
+   * @tparam Type to return the data as.
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, value, size.
+   * @return Value.
    */
-  tuple<Status, optional<void*>, optional<uint64_t>> get_tile_max(
-      const std::string& name, uint64_t tile_idx);
+  template <typename T>
+  T get_tile_max_as(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the tile sum value for a given attribute or dimension and tile
@@ -790,10 +786,9 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, sum.
+   * @return Sum.
    */
-  tuple<Status, optional<void*>> get_tile_sum(
-      const std::string& name, uint64_t tile_idx);
+  void* get_tile_sum(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the tile null count value for a given attribute or dimension
@@ -801,44 +796,41 @@ class FragmentMetadata {
    *
    * @param name The input attribute/dimension.
    * @param tile_idx The index of the tile in the metadata.
-   * @return Status, count.
+   * @return Count.
    */
-  tuple<Status, optional<uint64_t>> get_tile_null_count(
-      const std::string& name, uint64_t tile_idx);
+  uint64_t get_tile_null_count(const std::string& name, uint64_t tile_idx);
 
   /**
    * Retrieves the min value for a given attribute or dimension.
    *
    * @param name The input attribute/dimension.
-   * @return Status, value.
+   * @return Value.
    */
-  tuple<Status, optional<std::vector<uint8_t>>> get_min(
-      const std::string& name);
+  std::vector<uint8_t>& get_min(const std::string& name);
 
   /**
    * Retrieves the max value for a given attribute or dimension.
    *
    * @param name The input attribute/dimension.
-   * @return Status, value.
+   * @return Value.
    */
-  tuple<Status, optional<std::vector<uint8_t>>> get_max(
-      const std::string& name);
+  std::vector<uint8_t>& get_max(const std::string& name);
 
   /**
    * Retrieves the sum value for a given attribute or dimension.
    *
    * @param name The input attribute/dimension.
-   * @return Status, sum.
+   * @return Sum.
    */
-  tuple<Status, optional<void*>> get_sum(const std::string& name);
+  void* get_sum(const std::string& name);
 
   /**
    * Retrieves the null count value for a given attribute or dimension.
    *
    * @param name The input attribute/dimension.
-   * @return Status, count.
+   * @return Count.
    */
-  tuple<Status, optional<uint64_t>> get_null_count(const std::string& name);
+  uint64_t get_null_count(const std::string& name);
 
   /**
    * Set the processed conditions. The processed conditions is the list
@@ -2050,4 +2042,4 @@ class FragmentMetadata {
 }  // namespace sm
 }  // namespace tiledb
 
-#endif  // TILEDB_FRAGMENT_METADATA_H
+#endif  // TILEDB_FRAGMENT_METADATA_H
\ No newline at end of file
