@@ -51,6 +51,7 @@
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/query/validity_vector.h"
+#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 using namespace tiledb::common;
@@ -59,7 +60,7 @@ namespace tiledb {
 namespace sm {
 
 class Array;
-class StorageManager;
+class ArrayDimensionLabelQueries;
 
 enum class QueryStatus : uint8_t;
 enum class QueryType : uint8_t;
@@ -149,7 +150,7 @@ class Query {
       optional<std::string> fragment_name = nullopt);
 
   /** Destructor. */
-  ~Query();
+  virtual ~Query();
 
   DISABLE_COPY_AND_COPY_ASSIGN(Query);
   DISABLE_MOVE_AND_MOVE_ASSIGN(Query);
@@ -439,6 +440,26 @@ class Query {
   Status get_data_buffer(
       const char* name, void** buffer, uint64_t** buffer_size) const;
 
+  /**
+   * Retrieves the data buffer of a fixed or variable-sized dimension label.
+   *
+   * @param name The name of the the label.
+   * @param buffer The buffer to be retrieved.
+   * @param buffer_size The size of the buffer.
+   */
+  void get_label_data_buffer(
+      const std::string& name, void** buffer, uint64_t** buffer_size) const;
+
+  /**
+   * Retrieves the offset buffer for a variable-sized dimension label.
+   *
+   * @param name The name of the label.
+   * @param buffer The buffer to be retrieved.
+   * @param buffer_size The size of the buffer.
+   */
+  void get_label_offsets_buffer(
+      const std::string& name, uint64_t** buffer, uint64_t** buffer_size) const;
+
   /**
    * Retrieves the offset buffer for a var-sized attribute/dimension.
    *
@@ -570,6 +591,11 @@ class Query {
    */
   const std::vector<UpdateValue>& update_values() const;
 
+  /**
+   * Returns true if this query requires the use of dimension labels.
+   */
+  bool uses_dimension_labels() const;
+
   /** Processes a query. */
   Status process();
 
@@ -644,6 +670,51 @@ class Query {
       uint64_t* const buffer_size,
       const bool check_null_buffers = true);
 
+  /**
+   * Wrapper to set the internal buffer for a dimension or attribute from a
+   * QueryBuffer.
+   *
+   * This function is intended to be a convenience method for use setting
+   * buffers for dimension labels.
+   *
+   * @WARNING Does not check for or copy validity data.
+   *
+   * @param name Name of the dimension or attribute to set the buffer for.
+   * @param buffer The query buffer to get the data from.
+   **/
+  void set_dimension_label_buffer(
+      const std::string& name, const QueryBuffer& buffer);
+
+  /**
+   * Sets the label data buffer for fixed or variable sized dimension labels.
+   *
+   * @param name The name of the dimension label to set the data buffer for.
+   * @param buffer The buffer for the data.
+   * @param buffer_size The size of the data.
+   * @param check_null_buffers If ``true`` verify buffer and buffersize are not
+   *     nullptrs.
+   */
+  void set_label_data_buffer(
+      const std::string& name,
+      void* const buffer,
+      uint64_t* const buffer_size,
+      const bool check_null_buffers = true);
+
+  /**
+   * Sets the label offsets buffer for variable sized dimension labels.
+   *
+   * @param name The name of the dimension label to set the offsets buffer for.
+   * @param buffer_offsets The buffer for offsets for the variable size data.
+   * @param buffer_offsets_size The size of the offsets buffer.
+   * @param check_null_buffers If ``true`` verify buffer and buffersize are not
+   *     nullptrs.
+   */
+  void set_label_offsets_buffer(
+      const std::string& name,
+      uint64_t* const buffer_offsets,
+      uint64_t* const buffer_offsets_size,
+      const bool check_null_buffers = true);
+
   /**
    * Sets the offset buffer for a var-sized attribute/dimension.
    *
@@ -692,7 +763,7 @@ class Query {
    *
    * @return Config from query
    */
-  const Config* config() const;
+  const Config& config() const;
 
   /**
    * This is a deprecated API.
@@ -857,6 +928,21 @@ class Query {
       const void* update_value,
       uint64_t update_value_size);
 
+  /**
+   * Adds ranges to a query initialize with label ranges.
+   *
+   * @param dim_idx The dimension to update.
+   * @param is_point_ranges If ``true`` the data contains point ranges.
+   *     Otherwise, it contains standard ranges.
+   * @param start Pointer to the start of the range data.
+   * @param count Number of total elements in the range data.
+   */
+  void add_index_ranges_from_label(
+      uint32_t dim_idx,
+      const bool is_point_range,
+      const void* start,
+      const uint64_t count);
+
   /**
    * Set query status, needed for json deserialization
    * @param status
@@ -947,6 +1033,23 @@ class Query {
   /** Called from serialization to mark the query as remote */
   void set_remote_query();
 
+  /**
+   * Set a flag to specify we are doing an ordered dimension label read.
+   *
+   * @param increasing_order Is the query on an array with increasing order? If
+   * not assume decreasing order.
+   */
+  void set_dimension_label_ordered_read(bool increasing_order);
+
+  /**
+   * Set the fragment size.
+   *
+   * @param fragment_size Fragment size.
+   */
+  void set_fragment_size(uint64_t fragment_size) {
+    fragment_size_ = fragment_size;
+  }
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -1006,6 +1109,12 @@ class Query {
    * */
   std::unordered_map<std::string, QueryBuffer> buffers_;
 
+  /** Maps label names to their buffers. */
+  std::unordered_map<std::string, QueryBuffer> label_buffers_;
+
+  /** Dimension label queries that are part of the main query. */
+  tdb_unique_ptr<ArrayDimensionLabelQueries> dim_label_queries_;
+
   /** Keeps track of the coords data. */
   CoordsInfo coords_info_;
 
@@ -1084,6 +1193,25 @@ class Query {
   /** It tracks if this is a remote query */
   bool remote_query_;
 
+  /** Flag to specify we are doing a dimension label ordered read. */
+  bool is_dimension_label_ordered_read_;
+
+  /**
+   * Is the dimension label ordered read on an array with increasing order? If
+   * not assume decreasing order.
+   *
+   * NOTE: Only used when is_dimension_label_order_read_ == true.
+   */
+  bool dimension_label_increasing_;
+
+  /**
+   * The desired fragment size. The writer will create a new fragment once
+   * this size has been reached.
+   *
+   * Note: This is only used for global order writes.
+   */
+  uint64_t fragment_size_;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
@@ -1108,6 +1236,16 @@ class Query {
    */
   Status check_buffers_correctness();
 
+  /**
+   * Returns true if only querying dimension labels.
+   *
+   * The query will only query dimension labels if all the following are true:
+   * 1. At most one dimension buffer is set.
+   * 2. No attribute buffers are set.
+   * 3. At least one label buffer is set.
+   */
+  bool only_dim_label_query() const;
+
   /**
    * This is a deprecated API.
    * Internal routine for setting fixed-sized, nullable attribute buffers with
