@@ -268,8 +268,13 @@ Status SparseIndexReaderBase::load_initial_data() {
   }
 
   qc_loaded_attr_names_.reserve(qc_loaded_attr_names_set_.size());
+  std::vector<std::string> var_size_to_load;
   for (auto& name : qc_loaded_attr_names_set_) {
     qc_loaded_attr_names_.emplace_back(name);
+
+    if (array_schema_.var_size(name)) {
+      var_size_to_load.emplace_back(name);
+    }
   }
 
   // For easy reference.
@@ -325,7 +330,6 @@ Status SparseIndexReaderBase::load_initial_data() {
   const auto dim_num = array_schema_.dim_num();
   dim_names_.reserve(dim_num);
   is_dim_var_size_.reserve(dim_num);
-  std::vector<std::string> var_size_to_load;
   for (unsigned d = 0; d < dim_num; ++d) {
     dim_names_.emplace_back(array_schema_.dimension_ptr(d)->name());
     is_dim_var_size_.emplace_back(array_schema_.var_size(dim_names_[d]));
@@ -335,11 +339,13 @@ Status SparseIndexReaderBase::load_initial_data() {
   RETURN_CANCEL_OR_ERROR(load_tile_offsets(subarray_, dim_names_));
 
   // Compute tile offsets to load and var size to load for attributes.
-  std::vector<std::string> attr_tile_offsets_to_load;
+  std::vector<std::string> attr_tile_offsets_to_load(qc_loaded_attr_names_);
   for (auto& it : buffers_) {
     const auto& name = it.first;
-    if (array_schema_.is_dim(name))
+    if (array_schema_.is_dim(name) ||
+        qc_loaded_attr_names_set_.count(name) != 0) {
       continue;
+    }
 
     attr_tile_offsets_to_load.emplace_back(name);
 
