@@ -44,8 +44,6 @@
 #endif
 // clang-format on
 
-#include "tiledb/sm/query/ast/query_ast.h"
-#include "tiledb/sm/query/query.h"
 #include "tiledb/common/common.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
@@ -60,12 +58,14 @@
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/hash.h"
 #include "tiledb/sm/misc/parse_argument.h"
-#include "tiledb/sm/query/deletes_and_updates/deletes.h"
-#include "tiledb/sm/query/writers/global_order_writer.h"
-#include "tiledb/sm/query/readers/dense_reader.h"
+#include "tiledb/sm/query/ast/query_ast.h"
+#include "tiledb/sm/query/deletes_and_updates/deletes_and_updates.h"
 #include "tiledb/sm/query/legacy/reader.h"
+#include "tiledb/sm/query/query.h"
+#include "tiledb/sm/query/readers/dense_reader.h"
 #include "tiledb/sm/query/readers/sparse_global_order_reader.h"
 #include "tiledb/sm/query/readers/sparse_unordered_with_dups_reader.h"
+#include "tiledb/sm/query/writers/global_order_writer.h"
 #include "tiledb/sm/query/writers/writer_base.h"
 #include "tiledb/sm/serialization/config.h"
 #include "tiledb/sm/serialization/fragment_metadata.h"
@@ -338,7 +338,7 @@ Status subarray_partitioner_to_capnp(
 Status subarray_partitioner_from_capnp(
     Stats* query_stats,
     Stats* reader_stats,
-    const Config* config,
+    const Config& config,
     const Array* array,
     const capnp::SubarrayPartitioner::Reader& reader,
     SubarrayPartitioner* partitioner,
@@ -362,7 +362,7 @@ Status subarray_partitioner_from_capnp(
   Subarray subarray(array, layout, query_stats, dummy_logger, false);
   RETURN_NOT_OK(subarray_from_capnp(reader.getSubarray(), &subarray));
   *partitioner = SubarrayPartitioner(
-      config,
+      &config,
       subarray,
       memory_budget,
       memory_budget_var,
@@ -423,12 +423,12 @@ Status subarray_partitioner_from_capnp(
         partition_info_reader.getSubarray(), &partition_info->partition_));
 
     if (compute_current_tile_overlap) {
-      partition_info->partition_.precompute_tile_overlap(
+      throw_if_not_ok(partition_info->partition_.precompute_tile_overlap(
           partition_info->start_,
           partition_info->end_,
-          config,
+          &config,
           compute_tp,
-          true);
+          true));
     }
   }
 
@@ -691,7 +691,8 @@ static Status condition_ast_to_capnp(
 
     for (size_t i = 0; i < node->get_children().size(); ++i) {
       auto child_builder = children_builder[i];
-      condition_ast_to_capnp(node->get_children()[i], &child_builder);
+      throw_if_not_ok(
+          condition_ast_to_capnp(node->get_children()[i], &child_builder));
     }
 
     // Validate and store the query condition combination op.
@@ -1104,7 +1105,7 @@ Status dense_reader_from_capnp(
 Status delete_from_capnp(
     const capnp::Delete::Reader& delete_reader,
     Query* query,
-    Deletes* delete_strategy) {
+    DeletesAndUpdates* delete_strategy) {
   // Query condition
   if (delete_reader.hasCondition()) {
     auto condition_reader = delete_reader.getCondition();
@@ -1127,7 +1128,7 @@ Status delete_from_capnp(
 
 Status delete_to_capnp(
     const Query& query,
-    Deletes& delete_strategy,
+    DeletesAndUpdates& delete_strategy,
     capnp::Delete::Builder* delete_builder) {
   auto condition = query.condition();
   if (!condition.empty()) {
@@ -1240,6 +1241,12 @@ Status query_to_capnp(
   auto type = query.type();
   auto array = query.array();
 
+  if (query.uses_dimension_labels()) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot serialize; serialization is not yet supported for queries "
+        "using dimension labels."));
+  }
+
   if (array == nullptr) {
     return LOG_STATUS(
         Status_SerializationError("Cannot serialize; array is null."));
@@ -1349,14 +1356,14 @@ Status query_to_capnp(
     RETURN_NOT_OK(writer_to_capnp(query, *writer, &builder, client_side));
   } else {
     auto builder = query_builder->initDelete();
-    auto delete_strategy = dynamic_cast<Deletes*>(query.strategy(true));
+    auto delete_strategy =
+        dynamic_cast<DeletesAndUpdates*>(query.strategy(true));
     RETURN_NOT_OK(delete_to_capnp(query, *delete_strategy, &builder));
   }
 
   // Serialize Config
-  const Config* config = query.config();
   auto config_builder = query_builder->initConfig();
-  RETURN_NOT_OK(config_to_capnp(config, &config_builder));
+  RETURN_NOT_OK(config_to_capnp(query.config(), &config_builder));
 
   // If stats object exists set its cap'n proto object
   stats::Stats* stats = query.stats();
@@ -1960,7 +1967,7 @@ Status query_from_capnp(
     }
   } else {
     auto delete_reader = query_reader.getDelete();
-    auto delete_strategy = dynamic_cast<Deletes*>(query->strategy());
+    auto delete_strategy = dynamic_cast<DeletesAndUpdates*>(query->strategy());
     RETURN_NOT_OK(delete_from_capnp(delete_reader, query, delete_strategy));
   }
 
@@ -2411,9 +2418,9 @@ Status global_write_state_to_capnp(
 
   auto& cells_written = write_state.cells_written_;
   if (!cells_written.empty()) {
-    auto cells_writen_builder = state_builder->initCellsWritten();
+    auto cells_written_builder = state_builder->initCellsWritten();
     auto entries_builder =
-        cells_writen_builder.initEntries(cells_written.size());
+        cells_written_builder.initEntries(cells_written.size());
     uint64_t index = 0;
     for (auto& entry : cells_written) {
       entries_builder[index].setKey(entry.first);
