@@ -44,6 +44,7 @@
 #include "tiledb/sm/misc/tile_overlap.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/stats/stats.h"
+#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/range_subset.h"
 #include "tiledb/sm/subarray/subarray_tile_overlap.h"
 
@@ -71,7 +72,6 @@ class DimensionLabelReference;
 class EncryptionKey;
 class FragIdx;
 class FragmentMetadata;
-class StorageManager;
 
 enum class Layout : uint8_t;
 enum class QueryType : uint8_t;
@@ -250,6 +250,21 @@ class Subarray {
   /** equivalent for older Query::set_subarray(const void *subarray); */
   Status set_subarray(const void* subarray);
 
+  /**
+   * Adds dimension ranges computed from label ranges on the dimension label.
+   *
+   * @param dim_idx The dimension to add ranges to.
+   * @param is_point_ranges If ``true`` the data contains point ranges.
+   *     Otherwise, it contains standard ranges.
+   * @param start Pointer to the start of the range array.
+   * @param count Number of total elements in the range array.
+   */
+  void add_index_ranges_from_label(
+      const uint32_t dim_idx,
+      const bool is_point_ranges,
+      const void* start,
+      const uint64_t count);
+
   /**
    * Adds a range along the dimension with the given index for the requested
    * dimension label.
@@ -313,25 +328,40 @@ class Subarray {
   /**
    * @brief Set point ranges from an array
    *
-   * @param dim_idx Dimension index
-   * @param start Pointer to start of the array
-   * @param count Number of elements to add
+   * @param dim_idx Dimension index.
+   * @param start Pointer to start of the array.
+   * @param count Number of elements to add.
+   * @param check_for_label If ``true``, verify no label ranges set on this
+   *   dimension. This should check for labels unless being called by
+   *   ``add_index_ranges_from_label`` to update label ranges with index values.
    * @return Status
    */
-  Status add_point_ranges(unsigned dim_idx, const void* start, uint64_t count);
+  Status add_point_ranges(
+      unsigned dim_idx,
+      const void* start,
+      uint64_t count,
+      bool check_for_label = true);
 
   /**
    * @brief Set ranges from an array of ranges (paired { begin,end } )
    *
-   * @param dim_idx Dimension index
-   * @param start Pointer to start of the array
-   * @param count Number of pairs to add
+   * @param dim_idx Dimension index.
+   * @param start Pointer to start of the array.
+   * @param count Number of total elemenst to add. Must contain two elements for
+   *     each range.
+   * @param check_for_label If ``true``, verify no label ranges set on this
+   *   dimension. This should check for labels unless being called by
+   *   ``add_index_ranges_from_label`` to update label ranges with index values.
    * @return Status
    * @note The pairs list is logically { {begin1,end1}, {begin2,end2}, ...} but
    * because of typing considerations from the C api is simply presented as
    * a linear list of individual items, though they should be multiple of 2
    */
-  Status add_ranges_list(unsigned dim_idx, const void* start, uint64_t count);
+  Status add_ranges_list(
+      unsigned dim_idx,
+      const void* start,
+      uint64_t count,
+      bool check_for_label = true);
 
   /**
    * Adds a variable-sized range to the (read/write) query on the input
@@ -381,6 +411,13 @@ class Subarray {
   const std::vector<Range>& get_attribute_ranges(
       const std::string& attr_name) const;
 
+  /**
+   * Returns the name of the dimension label at the dimension index.
+   *
+   * @param dim_index Index of the dimension to return the label name for.
+   */
+  const std::string& get_label_name(const uint32_t dim_index) const;
+
   /**
    * Retrieves a range from a dimension label name in the form (start, end,
    * stride).
@@ -630,6 +667,9 @@ class Subarray {
   /** Returns the domain the subarray is constructed from. */
   NDRange domain() const;
 
+  /** ``True`` if the dimension of the subarray does not contain any ranges. */
+  bool empty(uint32_t dim_idx) const;
+
   /** ``True`` if the subarray does not contain any ranges. */
   bool empty() const;
 
@@ -637,7 +677,7 @@ class Subarray {
    * Retrieves a range of a given dimension at a given range index.
    *
    * @note Note that the retrieved range may be invalid if
-   *     Subarray::set_range() is called after this function. In that case,
+   *     Subarray::add_range() is called after this function. In that case,
    *     make sure to make a copy in the caller function.
    */
   Status get_range(
@@ -648,7 +688,7 @@ class Subarray {
    * The range is in the form (start, end).
    *
    * @note Note that the retrieved range may be invalid if
-   *     Subarray::set_range() is called after this function. In that case,
+   *     Subarray::add_range() is called after this function. In that case,
    *     make sure to make a copy in the caller function.
    */
   Status get_range(
@@ -862,6 +902,19 @@ class Subarray {
    */
   Subarray get_subarray(uint64_t start, uint64_t end) const;
 
+  /**
+   * Returns ``true`` if the any dimension in the subarray have label ranges
+   * set.
+   */
+  bool has_label_ranges() const;
+
+  /**
+   * Returns ``true`` if the dimension index has label ranges set.
+   *
+   * @param dim_idx The dimension index to check for ranges.
+   */
+  bool has_label_ranges(const uint32_t dim_index) const;
+
   /**
    * Set default indicator for dimension subarray. Used by serialization only
    * @param dim_index
@@ -933,6 +986,15 @@ class Subarray {
   void set_attribute_ranges(
       const std::string& attr_name, const std::vector<Range>& ranges);
 
+  /**
+   * Returns the `Range` vector for the given dimension label.
+   *
+   * @param label_name Name of the label to return ranges for.
+   * @returns Vector of ranges on the requested dimension label.
+   */
+  const std::vector<Range>& ranges_for_label(
+      const std::string& label_name) const;
+
   /**
    * Directly sets the `Range` vector for the given dimension index, making
    * a deep copy.
@@ -1190,6 +1252,10 @@ class Subarray {
     LabelRangeSubset(
         const DimensionLabelReference& ref, bool coalesce_ranges = true);
 
+    inline const std::vector<Range>& get_ranges() const {
+      return ranges.ranges();
+    }
+
     /** Name of the dimension label. */
     std::string name;
 
